
#ifndef PLATFORMSOFTWAREKEYBOARD_H
#define PLATFORMSOFTWAREKEYBOARD_H

#include <string>

namespace LibPlatform {

class SoftwareKeyboard {
public:
	typedef enum
	{
		DISPLAY_TV = 0,
		DISPLAY_GAMEPAD0,
		DISPLAY_GAMEPAD1,

		MAX_DISPLAYS
	} Display;

	typedef enum
	{
		MODE_FULL = 0,
		MODE_NUMERIC,
		MODE_QWERTY,
		MODE_HAND,

		MAX_MODES
	} Mode;

	typedef enum
	{
		PASSWORD_DISABLE = 0,
		PASSWORD_IMMEDIATE,
		PASSWORD_1SEC,

		MAX_PASSWORDMODES
	} PasswordMode;

	typedef enum
	{
		INVALIDCHAR_LINEFEED =	1 << 0,
		INVALIDCHAR_SPACE =		1 << 1,
		INVALIDCHAR_ATMARK =	1 << 2,
		INVALIDCHAR_PERCENT =	1 << 3,
		INVALIDCHAR_SLASH =		1 << 4,
		INVALIDCHAR_BACKSLASH =	1 << 5,
		INVALIDCHAR_NUMERIC =	1 << 6,
		INVALIDCHAR_HEART =		1 << 7,
		INVALIDCHAR_EUROMARK =	1 << 8,

		MAX_INVALIDCHARS = 9
	} InvalidCharFlags;

	typedef enum
	{
		INPUTFORMTYPE_MONOSPACE = 0,
		INPUTFORMTYPE_PROPORTIONAL,

		MAX_INPUTFORMTYPES
	} InputFormType;

	typedef enum
	{
		USERENTER_OK = 0,
		USERENTER_CANCEL,
		USERENTER_CHAR,
		USERENTER_BACKSPACE,

		MAX_USERENTERS
	} UserEnter;

	typedef enum
	{
		DISMISS_OK = 0,
		DISMISS_CANCEL,

		MAX_DISMISSOPTIONS
	} DismissOption;

	enum State
	{
		STATE_SHOW_HALFSCREEN = 0,
		STATE_SHOW_FULLSCREEN,
		STATE_HIDE,
		STATE_HIDDEN,
		STATE_KEY_DOWN,
		STATE_KEY_UP,
		STATE_KEY,
		STATE_OK,
		STATE_CANCEL,

		MAX_STATES
	};

	enum View
	{
		VIEW_NONE,
		VIEW_TV,
		VIEW_GAMEPAD,
		VIEW_UNKNOWN
	};

	class InvokeOptions
	{
	public:
		InvokeOptions() :	display(DISPLAY_GAMEPAD0)
						  , bFullscreen(false)
						  , initialString(L"")
						  , bPrediction(false)
						  , mode(MODE_FULL)
						  , password(PASSWORD_DISABLE)
						  , maxCharacters(-1)
						  , okString(L"")
						  , guideString(L"")
						  , formType(INPUTFORMTYPE_PROPORTIONAL)
						  , invalidChars(InvalidCharFlags(0))
		{}

		Display display;				// Specifies the display that the software keyboard should be displayed on.
		bool bFullscreen;				// Whether or not to display in full-screen mode.
		std::wstring initialString;		// Specifies the text that will first appear in the software keyboard input field.
		bool bPrediction;				// Whether or not to use predictive input.
		Mode mode;						// Keyboard mode (full, numeric, qwerty, etc.).
		PasswordMode password;			// Password mode (disabled, immediate, 1sec).
		int maxCharacters;				// The maximum number of characters the software keyboard will allow (-1 for no limit).
		std::wstring okString;			// The custom string to use for the OK button.
		std::wstring guideString;		// 
		InputFormType formType;			// Character spacing of the input form (only valid in fullscreen mode)
		InvalidCharFlags invalidChars;	// Bitmask for specifying which characters are invalid
	};

	typedef void (*StateChangeCallback)(State, std::wstring characters, View view, int index);

	static void Initialize(void *fs_client, bool isDefaultDisabled);
	static void Deinitialize(void);
	static SoftwareKeyboard *GetInstance(void);

	void AddGlobalCallback(StateChangeCallback callback);
	void RemoveGlobalCallback(StateChangeCallback callback);
	bool HasGlobalCallback(StateChangeCallback callback);
	/**
	@brief Returns whether or not the software keyboard is enabled.
	@return true if the default software keyboard in enabled.
	*/
	bool IsEnabled(void);

	/**
	@brief Returns which screen the software keyboard is being displayed on.
	@return The location that the software keyboard is being displayed on.
	*/
	Display GetDisplay(void);
		
	/**
	@brief Returns whether or not the given cursor position lies within the bounds of the keyboard.
	@param webviewHeight Height of the current webview. Valid values are 1080, 720, 480, and 'WUGP' (special value to be used for GamePad).
	@param cursorY The y-position of the cursor.
	@return true of the given cursor position lies within the bounds of the keyboard.
		    If the keyboard is covered by a sub-window, such as the window for switching keyboards,
			this will return true regardless of where the cursor position lies.
	*/
	bool IsKeyboardTarget(unsigned int webviewHeight, int cursorY);
		
	/**
	@brief Invokes the software keyboard.
	@param display Options for displaying the software keyboard.
	*/
	void Invoke(const InvokeOptions &options);
		
	/**
	@brief Dismisses the software keyboard.
	@param option Specifies whether to treat the dismissal as if the user pressed OK or Cancel.
	*/
	void Dismiss(DismissOption option);

	void SetText(std::wstring newText, int cursorPos = -1);
	void SetSelection(int cursorBegin, int cursorEnd);

	void SetKPADInfoPointer(void *kpad_status_pointer, int channel);
	void SetVPADInfoPointer(void *vpad_status_pointer);

	void SetEventRemote(int eventRemote);

	/**
	@brief Updates the software keyboard.
	*/
	void Update(int channel = 0);
		
	/**
	@brief Is the software keyboard currently being displayed?
	*/
	bool IsDisplaying(void);
		
	/**
	@brief Is the software keyboard currently in fullscreen mode?
	*/
	bool IsFullscreen(void);
		
	/**
	@brief This is a hack to keep the software keyboard drawing in the same rpl as all of its other functions until the software keyboard gets fixed.
	*/
	void DrawTV(void *pColorBuffer);
		
	/**
	@brief This is a hack to keep the software keyboard drawing in the same rpl as all of its other functions until the software keyboard gets fixed.
	*/
	void DrawGamePad(void *pColorBuffer);

private:
	SoftwareKeyboard(void *fs_client);
	~SoftwareKeyboard(void);

	void handleOkButton();
	void handleCancelButton();
};

}

#endif//PLATFORMSOFTWAREKEYBOARD_H
